==> cloud-5.1.1.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/molecules/formblocks/PermissionInputBlock.tsx <==
import SecurityGroupPermission from 'model/SecurityGroupPermission';
import { Button, Col, Row, Table } from 'react-bootstrap';
import { BsTrashFill } from 'react-icons/bs';

type FormItemInfo = {
  type: 'string',
  label: string,
  placeholder?: string,
  key: string,
} | {
  type: 'select',
  label: string,
  itemList: { value: string, label: string }[],
  key: string,
}

type FormInfo = {
  source: 'ip4' | 'ip6' | 'group',
  itemList: FormItemInfo[]
}

const FORM_INFO_LIST: FormInfo[] = [
  {
    source: 'ip4',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
      ] },
      { type: 'string', label: 'CIDR IP', placeholder: '0.0.0.0/0', key: 'cidr_ip' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
  {
    source: 'ip6',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
      ] },
      { type: 'string', label: 'CIDR IPv6', placeholder: '0.0.0.0/0', key: 'cidr_ip_v6' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
  {
    source: 'group',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
      ] },
      { type: 'string', label: 'Group ID', key: 'group_id' },
      { type: 'string', label: 'Peering status', key: 'peering_status' },
      { type: 'string', label: 'Group user ID', key: 'user_id' },
      { type: 'string', label: 'VPC ID', key: 'vpc_id' },
      { type: 'string', label: 'Peering connection ID', key: 'peering_connection_id' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
]

const StringInput = ({ label, placeholder, value, setValue }: {
  label: string,
  placeholder?: string,
  value: string,
  setValue: (s: string) => void,
}) => {
  return <Col sm={3}>
    <div className="form-item js-form-item p-0 w-100">
      <label className="control-label">{label}</label>
      <input className="form-text form-control" value={value} placeholder={placeholder}
        onChange={(e) => {
        setValue(e.currentTarget.value);
      }} />
    </div>
  </Col>;
}

const SelectInput = ({ label, itemList, value, setValue }: {
  label: string,
  itemList: { value: string, label: string}[],
  value: string,
  setValue: (s: string) => void,
}) => {
  return <Col sm={3}>
    <div className="form-item js-form-item p-0 w-100">
      <label className="control-label">{label}</label>
      <div className="select-wrapper">
        <select className="form-select form-control" value={value}
          onChange={(e) => {
            setValue(e.currentTarget.value);
          }}>
            {
              itemList.map((item) => {
                return <option key={item.value} value={item.value}>{item.label}</option>;
              })
            }
        </select>
      </div>
    </div>
  </Col>;
}

const PermissionForm = ({ value, setValue, cloudServiceProvider }: {
  value: SecurityGroupPermission,
  setValue: (s: SecurityGroupPermission) => void,
  cloudServiceProvider: string,
}) => {
  const template = FORM_INFO_LIST.filter((info) => info.source === value.source);
  if (template.length === 0) {
    return <></>;
  }

  if (cloudServiceProvider === 'openstack') {
    // Remove "All" (-1) from the list of "IP protocol".
    if (template[0].itemList.filter(item => item.key === "ip_protocol")[0].itemList !== undefined ) {
      template[0].itemList.filter(item => item.key === "ip_protocol")[0].itemList =
        template[0].itemList.filter(item => item.key === "ip_protocol")[0].itemList.filter((column) => column.value !== '-1');
    }
  }

  const itemCount = template[0].itemList.length;
  const width = 4;
  const rowCount = Math.ceil((itemCount + width - 1) / width);
  const setSingleValue = (key: string, s: string) => {
    const newValue: {[key: string]: any} = JSON.parse(JSON.stringify(value));
    newValue[key] = s;
    setValue(newValue as SecurityGroupPermission);
  }

  return <>
    {
      [...Array(rowCount)].map((_, i) => i).map((row) => {
        return <Row key={row} className="mb-3">
          {
            template[0].itemList.map((item, index) => {
              if ((index < row * width) || ((row + 1) * width <= index)) {
                return <></>;
              }
              switch (item.type) {
                case 'string':
                  return <StringInput label={item.label} placeholder={item.placeholder}
                    value={(value as {[key: string]: any})[item.key]}
                    setValue={(s) => setSingleValue(item.key, s)} />;
                case 'select':
                  return <SelectInput label={item.label} itemList={item.itemList}
                  value={(value as {[key: string]: any})[item.key]}
                  setValue={(s) => setSingleValue(item.key, s)} />;
                default:
                  return <></>;
              }
            })
          }
        </Row>;
      })
    }
  </>;
}

/**
 * Block of key-value input view.
 *
 * @param
 */
const PermissionInputBlock = ({ label, value, setValue, cloudServiceProvider }: {
  label: string,
  value: SecurityGroupPermission[],
  setValue: (s: SecurityGroupPermission[]) => void,
  cloudServiceProvider: string,
}) => {

  const setSingleValue = (sg: SecurityGroupPermission, index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    newValue[index] = sg;
    setValue(newValue);
  }

  const addPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    newValue.push({
      cidr_ip: '',
      cidr_ip_v6: '',
      description: '',
      from_port: '',
      group_id: '',
      group_name: '',
      ip_protocol: '-1',
      peering_connection_id: '',
      peering_status: '',
      prefix_list_id: '',
      rule_id: '',
      source: 'ip4',
      to_port: '',
      user_id: '',
      vpc_id: '',
    });
    setValue(newValue);
  }

  const moveUpPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const value1 = JSON.parse(JSON.stringify(newValue[index - 1]));
    const value2 = JSON.parse(JSON.stringify(newValue[index]));
    newValue[index] = value1;
    newValue[index - 1] = value2;
    setValue(newValue);
  }

  const moveDownPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const value1 = JSON.parse(JSON.stringify(newValue[index + 1]));
    const value2 = JSON.parse(JSON.stringify(newValue[index]));
    newValue[index] = value1;
    newValue[index + 1] = value2;
    setValue(newValue);
  }

  const deletePermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const newValue2: SecurityGroupPermission[] = [];
    for (let i = 0; i < newValue.length; i++) {
      if (i !== index) {
        newValue2.push(newValue[i]);
      }
    }
    setValue(newValue2);
  }

  return <div className="field--type-ip-permission mb-3">
    <div>
      <div>
        <Table responsive hover striped>
          <thead>
            <tr>
              <th className="field-label">
                <h4 className="label">{label}</h4>
              </th>
              <th>Operation</th>
            </tr>
          </thead>
          <tbody>
            {
              value && value.map((sg, index) => {
                return <tr key={index}>
                  <div className="field-multiple-drag"></div>
                  <td className="d-block">
                    <PermissionForm value={sg} setValue={(data) => setSingleValue(data, index)} cloudServiceProvider={cloudServiceProvider} />
                  </td>
                  <td>
                    <Button className="mx-1" onClick={() => addPermission(index)}>＋</Button>
                    <Button className="mx-1" disabled={index <= 0} onClick={() => moveUpPermission(index)}>↑</Button>
                    <Button className="mx-1" disabled={index >= value.length - 1} onClick={() => moveDownPermission(index)}>↓</Button>
                    <Button className="mx-1" onClick={() => deletePermission(index)}><BsTrashFill /></Button>
                  </td>
                </tr>;
              })
            }
          </tbody>
        </Table>
      </div>
    </div>
  </div>;

}

export default PermissionInputBlock;

==> cloud-5.2.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/molecules/formblocks/PermissionInputBlock.tsx <==
import Glyphicon from 'atoms/Glyphicon';
import SecurityGroupPermission from 'model/SecurityGroupPermission';
import { Button, Col, Row, Table } from 'react-bootstrap';

type FormItemInfo = {
  type: 'string',
  label: string,
  placeholder?: string,
  key: string,
} | {
  type: 'select',
  label: string,
  itemList: { value: string, label: string }[],
  key: string,
}

type FormInfo = {
  source: 'ip4' | 'ip6' | 'group' | 'prefix',
  itemList: FormItemInfo[]
}

const FORM_INFO_LIST: FormInfo[] = [
  {
    source: 'ip4',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
        { value: 'prefix', label: 'Prefix list Id' },
      ] },
      { type: 'string', label: 'CIDR IP', placeholder: '0.0.0.0/0', key: 'cidr_ip' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
  {
    source: 'ip6',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
        { value: 'prefix', label: 'Prefix list Id' },
      ] },
      { type: 'string', label: 'CIDR IPv6', placeholder: '0.0.0.0/0', key: 'cidr_ip_v6' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
  {
    source: 'group',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
        { value: 'prefix', label: 'Prefix list Id' },
      ] },
      { type: 'string', label: 'Group ID', key: 'group_id' },
      { type: 'string', label: 'Peering status', key: 'peering_status' },
      { type: 'string', label: 'Group user ID', key: 'user_id' },
      { type: 'string', label: 'VPC ID', key: 'vpc_id' },
      { type: 'string', label: 'Peering connection ID', key: 'peering_connection_id' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  },
  {
    source: 'prefix',
    itemList: [
      { type: 'select', label: 'IP protocol', key: 'ip_protocol', itemList: [
        { value: '-1', label: 'All' },
        { value: 'tcp', label: 'TCP' },
        { value: 'udp', label: 'UDP' },
        { value: 'icmp', label: 'ICMP' },
        { value: 'icmpv6', label: 'ICMPv6' },
      ] },
      { type: 'string', label: 'From port', placeholder: '1', key: 'from_port' },
      { type: 'string', label: 'To port', placeholder: '65535', key: 'to_port' },
      { type: 'select', label: 'Source', key: 'source', itemList: [
        { value: 'ip4', label: 'IP' },
        { value: 'ip6', label: 'IPv6' },
        { value: 'group', label: 'Group' },
        { value: 'prefix', label: 'Prefix list Id' },
      ] },
      { type: 'string', label: 'Prefix list ID', placeholder: '', key: 'prefix_list_id' },
      { type: 'string', label: 'Description', key: 'description' },
      { type: 'string', label: 'Rule ID', placeholder: '0', key: 'rule_id' },
    ]
  }
]

const StringInput = ({ label, placeholder, value, setValue, id, className }: {
  label: string,
  placeholder?: string,
  value: string,
  setValue: (s: string) => void,
  id?: string,
  className?: string,
}) => {
  return <Col sm={3}>
    <div className={"form-item js-form-item p-0 w-100" + (className ? ' ' + className : '')}>
      <label htmlFor={id} className="control-label">{label}</label>
      <input className="form-text form-control" value={value} id={id} placeholder={placeholder}
        onChange={(e) => {
        setValue(e.currentTarget.value);
      }} />
    </div>
  </Col>;
}

const SelectInput = ({ label, itemList, value, setValue, id, className }: {
  label: string,
  itemList: { value: string, label: string}[],
  value: string,
  setValue: (s: string) => void,
  id?: string,
  className?: string,
}) => {
  return <Col sm={3}>
    <div className={"form-item js-form-item p-0 w-100" + (className ? ' ' + className : '')}>
      <label htmlFor={id} className="control-label">{label}</label>
      <div className="select-wrapper">
        <select className="form-select form-control" id={id} value={value}
          onChange={(e) => {
            setValue(e.currentTarget.value);
          }}>
            {
              itemList.map((item) => {
                return <option key={item.value} value={item.value}>{item.label}</option>;
              })
            }
        </select>
      </div>
    </div>
  </Col>;
}

const PermissionForm = ({ value, setValue, cloudServiceProvider, blockIndex, id, className }: {
  value: SecurityGroupPermission,
  setValue: (s: SecurityGroupPermission) => void,
  cloudServiceProvider: string,
  blockIndex: number,
  id?: string,
  className?: string,
}) => {
  const template = FORM_INFO_LIST.filter((info) => info.source === value.source);
  if (template.length === 0) {
    return <></>;
  }

  const itemCount = template[0].itemList.length;
  const width = 4;
  const rowCount = Math.ceil((itemCount + width - 1) / width);
  const setSingleValue = (key: string, s: string) => {
    const newValue: {[key: string]: any} = JSON.parse(JSON.stringify(value));
    newValue[key] = s;
    setValue(newValue as SecurityGroupPermission);
  }

  return <>
    {
      [...Array(rowCount)].map((_, i) => i).map((row) => {
        return <Row key={row} className="mb-3">
          {
            template[0].itemList.map((item, index) => {
              if ((index < row * width) || ((row + 1) * width <= index)) {
                return <></>;
              }
              const formId = id ? `${id}-${blockIndex}-${item.key.replaceAll('_', '-')}` : undefined;
              const formClass = className ? `${className}-${blockIndex}-${item.key.replaceAll('_', '-')}` : undefined;
              switch (item.type) {
                case 'string':
                  return <StringInput label={item.label} placeholder={item.placeholder}
                    value={(value as {[key: string]: any})[item.key]}
                    setValue={(s) => setSingleValue(item.key, s)}
                    id={formId} className={formClass} />;
                case 'select':
                  const itemList = cloudServiceProvider === 'openstack' && item.key === 'ip_protocol'
                    ? item.itemList.filter((column) => column.value !== '-1')
                    : item.itemList;
                  return <SelectInput label={item.label} itemList={itemList}
                    value={(value as {[key: string]: any})[item.key]}
                    setValue={(s) => setSingleValue(item.key, s)}
                    id={formId} className={formClass} />;
                default:
                  return <></>;
              }
            })
          }
        </Row>;
      })
    }
  </>;
}

/**
 * Block of key-value input view.
 *
 * @param
 */
const PermissionInputBlock = ({ label, value, setValue, cloudServiceProvider, id, className }: {
  label: string,
  value: SecurityGroupPermission[],
  setValue: (s: SecurityGroupPermission[]) => void,
  cloudServiceProvider: string,
  id?: string,
  className?: string,
}) => {

  const setSingleValue = (sg: SecurityGroupPermission, index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    newValue[index] = sg;
    setValue(newValue);
  }

  const addPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    newValue.push({
      cidr_ip: '',
      cidr_ip_v6: '',
      description: '',
      from_port: '',
      group_id: '',
      group_name: '',
      ip_protocol: '-1',
      peering_connection_id: '',
      peering_status: '',
      prefix_list_id: '',
      rule_id: '',
      source: 'ip4',
      to_port: '',
      user_id: '',
      vpc_id: '',
    });
    setValue(newValue);
  }

  const moveUpPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const value1 = JSON.parse(JSON.stringify(newValue[index - 1]));
    const value2 = JSON.parse(JSON.stringify(newValue[index]));
    newValue[index] = value1;
    newValue[index - 1] = value2;
    setValue(newValue);
  }

  const moveDownPermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const value1 = JSON.parse(JSON.stringify(newValue[index + 1]));
    const value2 = JSON.parse(JSON.stringify(newValue[index]));
    newValue[index] = value1;
    newValue[index + 1] = value2;
    setValue(newValue);
  }

  const deletePermission = (index: number) => {
    const newValue: SecurityGroupPermission[] = JSON.parse(JSON.stringify(value));
    const newValue2: SecurityGroupPermission[] = [];
    for (let i = 0; i < newValue.length; i++) {
      if (i !== index) {
        newValue2.push(newValue[i]);
      }
    }
    setValue(newValue2);
  }

  return <div className="field--type-ip-permission mb-3" id={id ? id + '-wrapper' : ''}>
    <div>
      <div>
        <Table responsive hover striped>
          <thead>
            <tr>
              <th className="field-label">
                <h4 className="label">{label}</h4>
              </th>
              <th>Operation</th>
            </tr>
          </thead>
          <tbody>
            {
              value && value.map((sg, index) => {
                return <tr key={index}>
                  <div className="field-multiple-drag"></div>
                  <td className="d-block">
                    <PermissionForm value={sg} setValue={(data) => {
                      setSingleValue(data, index);
                    }} cloudServiceProvider={cloudServiceProvider} blockIndex={index} id={id} className={className} />
                  </td>
                  <td>
                    <Button className="mx-1" onClick={() => addPermission(index)}>
                      <Glyphicon type="plus" unMargin />
                    </Button>
                    <Button className="mx-1" disabled={index <= 0} onClick={() => moveUpPermission(index)}>
                      <Glyphicon type="chevron-up" unMargin />
                    </Button>
                    <Button className="mx-1" disabled={index >= value.length - 1} onClick={() => moveDownPermission(index)}>
                      <Glyphicon type="chevron-down" unMargin />
                    </Button>
                    <Button className="mx-1" onClick={() => deletePermission(index)}>
                      <Glyphicon type="trash" unMargin />
                    </Button>
                  </td>
                </tr>;
              })
            }
          </tbody>
        </Table>
      </div>
    </div>
  </div>;

}

export default PermissionInputBlock;
